=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2010-01-02 18:59:12 +0000
+++ drizzled/db.cc	2010-01-22 00:53:13 +0000
@@ -59,6 +59,7 @@
 static long mysql_rm_known_files(Session *session, CachedDirectory &dirp,
                                  const string &db, const char *path,
                                  TableList **dropped_tables);
+static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name);
 
 /**
   Return default database collation.
@@ -100,12 +101,12 @@
 }
 
 /* path is path to database, not schema file */
-static int write_schema_file(const DatabasePathName &path, const message::Schema &db)
+static int write_schema_file(const char *path, const message::Schema &db)
 {
   char schema_file_tmp[FN_REFLEN];
-  string schema_file(path.to_string());
+  string schema_file(path);
 
-  snprintf(schema_file_tmp, FN_REFLEN, "%s%c%s.tmpXXXXXX", path.to_string().c_str(), FN_LIBCHAR, MY_DB_OPT_FILE);
+  snprintf(schema_file_tmp, FN_REFLEN, "%s%c%s.tmpXXXXXX", path, FN_LIBCHAR, MY_DB_OPT_FILE);
 
   schema_file.append(1, FN_LIBCHAR);
   schema_file.append(MY_DB_OPT_FILE);
@@ -182,23 +183,21 @@
 
 */
 
-bool mysql_create_db(Session *session, const NormalisedDatabaseName &database_name, message::Schema *schema_message, bool is_if_not_exists)
+bool mysql_create_db(Session *session, const char *db, drizzled::message::Schema *schema_message, bool is_if_not_exists)
 {
   ReplicationServices &replication_services= ReplicationServices::singleton();
   long result= 1;
   int error_erno;
   bool error= false;
-  DatabasePathName database_path(database_name);
 
   /* do not create 'information_schema' db */
-  if (!my_strcasecmp(system_charset_info, database_name.to_string().c_str(), INFORMATION_SCHEMA_NAME.c_str()))
+  if (!my_strcasecmp(system_charset_info, db, INFORMATION_SCHEMA_NAME.c_str()))
   {
-    my_error(ER_DB_CREATE_EXISTS, MYF(0), database_name.to_string().c_str());
+    my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
     return(-1);
   }
 
-  assert(database_name.isValid());
-  schema_message->set_name(database_name.to_string());
+  schema_message->set_name(db);
 
   /*
     Do not create database if another thread is holding read lock.
@@ -220,33 +219,39 @@
 
   pthread_mutex_lock(&LOCK_create_db);
 
-  if (mkdir(database_path.to_string().c_str(),0777) == -1)
+  /* check directory */
+  char	 path[FN_REFLEN+16];
+  uint32_t path_len;
+  path_len= build_table_filename(path, sizeof(path), db, "", false);
+  path[path_len-1]= 0;                    // remove last '/' from path
+
+  if (mkdir(path, 0777) == -1)
   {
     if (errno == EEXIST)
     {
       if (! is_if_not_exists)
       {
-	my_error(ER_DB_CREATE_EXISTS, MYF(0), database_name.to_string().c_str());
+	my_error(ER_DB_CREATE_EXISTS, MYF(0), path);
 	error= true;
 	goto exit;
       }
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
 			  ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS),
-                          database_name.to_string().c_str());
+                          path);
       session->my_ok();
       error= false;
       goto exit;
     }
 
-    my_error(ER_CANT_CREATE_DB, MYF(0), database_name.to_string().c_str(), errno);
+    my_error(ER_CANT_CREATE_DB, MYF(0), path, errno);
     error= true;
     goto exit;
   }
 
-  error_erno= write_schema_file(database_path, *schema_message);
+  error_erno= write_schema_file(path, *schema_message);
   if (error_erno && error_erno != EEXIST)
   {
-    if (rmdir(database_path.to_string().c_str()) >= 0)
+    if (rmdir(path) >= 0)
     {
       error= true;
       goto exit;
@@ -268,12 +273,13 @@
 
 /* db-name is already validated when we come here */
 
-bool mysql_alter_db(Session *session, const NormalisedDatabaseName &database_name, message::Schema *schema_message)
+bool mysql_alter_db(Session *session, const char *db, message::Schema *schema_message)
 {
   ReplicationServices &replication_services= ReplicationServices::singleton();
   long result=1;
   int error= 0;
-  DatabasePathName database_path(database_name);
+  char	 path[FN_REFLEN+16];
+  uint32_t path_len;
 
   /*
     Do not alter database if another thread is holding read lock.
@@ -291,13 +297,16 @@
     goto exit;
 
   assert(schema_message);
-  assert(database_name.isValid());
 
-  schema_message->set_name(database_name.to_string());
+  schema_message->set_name(db);
 
   pthread_mutex_lock(&LOCK_create_db);
 
-  error= write_schema_file(database_path, *schema_message);
+  /* Change options if current database is being altered. */
+  path_len= build_table_filename(path, sizeof(path), db, "", false);
+  path[path_len-1]= 0;                    // Remove last '/' from path
+
+  error= write_schema_file(path, *schema_message);
   if (error && error != EEXIST)
   {
     /* TODO: find some witty way of getting back an error message */
@@ -332,7 +341,7 @@
     ERROR Error
 */
 
-bool mysql_rm_db(Session *session, const NormalisedDatabaseName &database_name, bool if_exists)
+bool mysql_rm_db(Session *session, char *db, bool if_exists)
 {
   long deleted=0;
   int error= false;
@@ -340,12 +349,13 @@
   uint32_t length;
   TableList *dropped_tables= NULL;
 
-  if (database_name.to_string().compare(INFORMATION_SCHEMA_NAME) == 0)
+  if (db && (strcmp(db, "information_schema") == 0))
   {
     my_error(ER_DBACCESS_DENIED_ERROR, MYF(0), "", "", INFORMATION_SCHEMA_NAME.c_str());
     return true;
   }
 
+
   /*
     Do not drop database if another thread is holding read lock.
     Wait for global read lock before acquiring LOCK_create_db.
@@ -366,7 +376,7 @@
   pthread_mutex_lock(&LOCK_create_db);
 
   length= build_table_filename(path, sizeof(path),
-                               database_name.to_string().c_str(), "", false);
+                               db, "", false);
   strcpy(path+length, MY_DB_OPT_FILE);         // Append db option file name
   unlink(path);
   path[length]= '\0';				// Remove file name
@@ -378,24 +388,23 @@
     if (!if_exists)
     {
       error= -1;
-      my_error(ER_DB_DROP_EXISTS, MYF(0), database_name.to_string().c_str());
+      my_error(ER_DB_DROP_EXISTS, MYF(0), path);
       goto exit;
     }
     else
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
 			  ER_DB_DROP_EXISTS, ER(ER_DB_DROP_EXISTS),
-                          database_name.to_string().c_str());
+                          path);
   }
   else
   {
     pthread_mutex_lock(&LOCK_open); /* After deleting database, remove all cache entries related to schema */
-    remove_db_from_cache(database_name.to_string().c_str());
+    remove_db_from_cache(db);
     pthread_mutex_unlock(&LOCK_open);
 
 
     error= -1;
-    deleted= mysql_rm_known_files(session, dirp,
-                                  database_name.to_string(),
+    deleted= mysql_rm_known_files(session, dirp, db,
                                   path, &dropped_tables);
     if (deleted >= 0)
     {
@@ -430,7 +439,7 @@
       goto exit; /* not much else we can do */
     query_pos= query_data_start= strcpy(query,"drop table ")+11;
     query_end= query + MAX_DROP_TABLE_Q_LEN;
-    db_len= database_name.to_string().length();
+    db_len= strlen(db);
 
     ReplicationServices &replication_services= ReplicationServices::singleton();
     for (tbl= dropped_tables; tbl; tbl= tbl->next_local)
@@ -466,8 +475,8 @@
     SELECT DATABASE() in the future). For this we free() session->db and set
     it to 0.
   */
-  if (! session->db.empty() && session->db.compare(database_name.to_string()) == 0)
-    session->clear_db();
+  if (! session->db.empty() && session->db.compare(db) == 0)
+    mysql_change_db_impl(session, NULL);
   pthread_mutex_unlock(&LOCK_create_db);
   start_waiting_global_read_lock(session);
   return error;
@@ -784,21 +793,42 @@
     @retval true  Error
 */
 
-bool mysql_change_db(Session *session, const NormalisedDatabaseName &normalised_database_name, bool force_switch)
+bool mysql_change_db(Session *session, const LEX_STRING *new_db_name, bool force_switch)
 {
+  LEX_STRING new_db_file_name;
   const CHARSET_INFO *db_default_cl;
 
-  if (my_strcasecmp(system_charset_info, normalised_database_name.to_string().c_str(),
+  assert(new_db_name);
+  assert(new_db_name->length);
+
+  if (my_strcasecmp(system_charset_info, new_db_name->str,
                     INFORMATION_SCHEMA_NAME.c_str()) == 0)
   {
-    NonNormalisedDatabaseName non_normalised_i_s(INFORMATION_SCHEMA_NAME);
-    NormalisedDatabaseName is_name(non_normalised_i_s);
+    /* Switch the current database to INFORMATION_SCHEMA. */
+    /* const_cast<> is safe here: mysql_change_db_impl does a copy */
+    LEX_STRING is_name= { const_cast<char *>(INFORMATION_SCHEMA_NAME.c_str()),
+                          INFORMATION_SCHEMA_NAME.length() };
+    mysql_change_db_impl(session, &is_name);
 
-    session->set_db(is_name);
     return false;
   }
 
   /*
+    Now we need to make a copy because check_db_name requires a
+    non-constant argument. Actually, it takes database file name.
+
+    TODO: fix check_db_name().
+  */
+
+  new_db_file_name.length= new_db_name->length;
+  new_db_file_name.str= (char *)malloc(new_db_name->length + 1);
+  if (new_db_file_name.str == NULL)
+    return true;                             /* the error is set */
+  memcpy(new_db_file_name.str, new_db_name->str, new_db_name->length);
+  new_db_file_name.str[new_db_name->length]= 0;
+
+
+  /*
     NOTE: if check_db_name() fails, we should throw an error in any case,
     even if we are called from sp_head::execute().
 
@@ -806,20 +836,19 @@
     from sp_head::execute(). But let's switch the current database to NULL
     in this case to be sure.
   */
-  if (! normalised_database_name.isValid())
+
+  if (check_db_name(&new_db_file_name))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0),
-             normalised_database_name.to_string().c_str());
+    my_error(ER_WRONG_DB_NAME, MYF(0), new_db_file_name.str);
+    free(new_db_file_name.str);
 
     if (force_switch)
-      session->clear_db();
+      mysql_change_db_impl(session, NULL);
 
     return true;
   }
 
-  DatabasePathName database_path(normalised_database_name);
-
-  if (! database_path.exists())
+  if (check_db_dir_existence(new_db_file_name.str))
   {
     if (force_switch)
     {
@@ -827,11 +856,13 @@
 
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
                           ER_BAD_DB_ERROR, ER(ER_BAD_DB_ERROR),
-                          normalised_database_name.to_string().c_str());
+                          new_db_file_name.str);
+
+      free(new_db_file_name.str);
 
       /* Change db to NULL. */
 
-      session->clear_db();
+      mysql_change_db_impl(session, NULL);
 
       /* The operation succeed. */
 
@@ -841,8 +872,8 @@
     {
       /* Report an error and free new_db_file_name. */
 
-      my_error(ER_BAD_DB_ERROR, MYF(0),
-               normalised_database_name.to_string().c_str());
+      my_error(ER_BAD_DB_ERROR, MYF(0), new_db_file_name.str);
+      free(new_db_file_name.str);
 
       /* The operation failed. */
 
@@ -850,62 +881,74 @@
     }
   }
 
-  db_default_cl= get_default_db_collation(normalised_database_name.to_string().c_str());
+  db_default_cl= get_default_db_collation(new_db_file_name.str);
 
-  session->set_db(normalised_database_name);
+  mysql_change_db_impl(session, &new_db_file_name);
+  free(new_db_file_name.str);
 
   return false;
 }
 
-NormalisedDatabaseName::NormalisedDatabaseName(const NonNormalisedDatabaseName &dbname)
-{
-  const std::string &non_norm_string= dbname.to_string();
-  database_name= (char*)malloc(non_norm_string.size()+1);
-
-  assert(database_name); /* FIXME: should throw exception */
-
-  strncpy(database_name, non_norm_string.c_str(), non_norm_string.size()+1);
-
-  my_casedn_str(files_charset_info, database_name);
-}
-
-NormalisedDatabaseName::~NormalisedDatabaseName()
-{
-  free(database_name);
-}
-
-bool NormalisedDatabaseName::isValid() const
-{
-  LEX_STRING db_lexstring;
-
-  db_lexstring.str= database_name;
-  db_lexstring.length= strlen(database_name);
-
-  if (db_lexstring.length == 0
-      || db_lexstring.length > NAME_LEN
-      || database_name[db_lexstring.length - 1] == ' ')
-    return false;
-
-  return (! check_identifier_name(&db_lexstring));
-}
-
-DatabasePathName::DatabasePathName(const NormalisedDatabaseName &database_name)
+/*
+  Check if there is directory for the database name.
+
+  SYNOPSIS
+    check_db_dir_existence()
+    db_name   database name
+
+  RETURN VALUES
+    false   There is directory for the specified database name.
+    true    The directory does not exist.
+*/
+
+bool check_db_dir_existence(const char *db_name)
 {
   char db_dir_path[FN_REFLEN];
   uint32_t db_dir_path_len;
 
   db_dir_path_len= build_table_filename(db_dir_path, sizeof(db_dir_path),
-                                        database_name.to_string().c_str(),
-                                        "", false);
+                                        db_name, "", false);
 
   if (db_dir_path_len && db_dir_path[db_dir_path_len - 1] == FN_LIBCHAR)
     db_dir_path[db_dir_path_len - 1]= 0;
 
-  database_path.assign(db_dir_path);
+  /* Check access. */
+
+  return access(db_dir_path, F_OK);
 }
 
-bool DatabasePathName::exists() const
+/**
+  @brief Internal implementation: switch current database to a valid one.
+
+  @param session            Thread context.
+  @param new_db_name    Name of the database to switch to. The function will
+                        take ownership of the name (the caller must not free
+                        the allocated memory). If the name is NULL, we're
+                        going to switch to NULL db.
+  @param new_db_charset Character set of the new database.
+*/
+
+static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name)
 {
-  /* TODO: handle EIO and other fun errors */
-  return access(database_path.c_str(), F_OK) == 0;
+  /* 1. Change current database in Session. */
+
+  if (new_db_name == NULL)
+  {
+    /*
+      Session::set_db() does all the job -- it frees previous database name and
+      sets the new one.
+    */
+
+    session->set_db(NULL, 0);
+  }
+  else
+  {
+    /*
+      Here we already have a copy of database name to be used in Session. So,
+      we just call Session::reset_db(). Since Session::reset_db() does not releases
+      the previous database name, we should do it explicitly.
+    */
+
+    session->set_db(new_db_name->str, new_db_name->length);
+  }
 }

=== modified file 'drizzled/db.h'
--- drizzled/db.h	2009-12-08 01:43:27 +0000
+++ drizzled/db.h	2010-01-22 00:53:13 +0000
@@ -23,84 +23,14 @@
 
 namespace drizzled { namespace message { class Schema; } }
 
-class NormalisedDatabaseName;
-
-bool mysql_create_db(Session *session, const NormalisedDatabaseName &database_name, drizzled::message::Schema *schema_message, bool is_if_not_exists);
-bool mysql_alter_db(Session *session, const NormalisedDatabaseName &database_name, drizzled::message::Schema *schema_message);
-bool mysql_rm_db(Session *session, const NormalisedDatabaseName &database_name, bool if_exists);
-bool mysql_change_db(Session *session, const NormalisedDatabaseName &new_db_name, bool force_switch);
+bool mysql_create_db(Session *session, const char *db, drizzled::message::Schema *schema_message, bool is_if_not_exists);
+bool mysql_alter_db(Session *session, const char *db, drizzled::message::Schema *schema_message);
+bool mysql_rm_db(Session *session, char *db, bool if_exists);
+bool mysql_change_db(Session *session, const LEX_STRING *new_db_name, bool force_switch);
 
 bool check_db_dir_existence(const char *db_name);
 int get_database_metadata(const char *dbname, drizzled::message::Schema *db);
 
 const CHARSET_INFO *get_default_db_collation(const char *db_name);
 
-class NonNormalisedDatabaseName
-{
-private:
-  std::string database_name;
-
-  /* Copying a NonNormalisedDatabaseName is always wrong, it's
-     immutable and should be passed by reference */
-  NonNormalisedDatabaseName(const NonNormalisedDatabaseName&);
-
-  NonNormalisedDatabaseName operator=(const NonNormalisedDatabaseName&);
-
-public:
-  explicit NonNormalisedDatabaseName(const std::string db) :
-    database_name(db)
-  {
-  }
-
-  const std::string &to_string(void) const
-  {
-    return database_name;
-  }
-};
-
-class NormalisedDatabaseName
-{
-private:
-  char* database_name;
-
-  /* Copying a NormalisedDatabaseName is always wrong, it's
-     immutable and should be passed by reference */
-  NormalisedDatabaseName(const NormalisedDatabaseName&);
-  NormalisedDatabaseName operator=(const NormalisedDatabaseName&);
-
-public:
-  explicit NormalisedDatabaseName(const NonNormalisedDatabaseName &dbname);
-
-  ~NormalisedDatabaseName();
-
-  const std::string to_string() const
-  {
-    std::string tmp(database_name);
-    return tmp;
-  }
-
-  bool isValid() const;
-};
-
-class DatabasePathName
-{
-private:
-  std::string database_path;
-
-  /* Copying a DatabasePathName is always wrong, it's
-     immutable and should be passed by reference. */
-  DatabasePathName(const DatabasePathName&);
-  DatabasePathName operator=(const DatabasePathName&);
-
-public:
-  explicit DatabasePathName(const NormalisedDatabaseName &database_name);
-
-  const std::string to_string() const
-  {
-    return database_path;
-  }
-
-  bool exists() const;
-};
-
 #endif /* DRIZZLED_DB_H */

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-01-02 23:27:00 +0000
+++ drizzled/session.cc	2010-01-22 00:53:13 +0000
@@ -677,6 +677,7 @@
 
 bool Session::checkUser(const char *passwd, uint32_t passwd_len, const char *in_db)
 {
+  LEX_STRING db_str= { (char *) in_db, in_db ? strlen(in_db) : 0 };
   bool is_authenticated;
 
   if (passwd_len != 0 && passwd_len != SCRAMBLE_LENGTH)
@@ -702,11 +703,7 @@
   /* Change database if necessary */
   if (in_db && in_db[0])
   {
-    const string database_name_string(in_db);
-    NonNormalisedDatabaseName database_name(database_name_string);
-    NormalisedDatabaseName normalised_database_name(database_name);
-
-    if (mysql_change_db(this, normalised_database_name, false))
+    if (mysql_change_db(this, &db_str, false))
     {
       /* mysql_change_db() has pushed the error message. */
       return false;
@@ -1729,18 +1726,18 @@
   set_open_tables_state(backup);
 }
 
-
-bool Session::set_db(const NormalisedDatabaseName &new_db)
+bool Session::set_db(const char *new_db, size_t length)
 {
-  db= new_db.to_string();
+  /* Do not reallocate memory if current chunk is big enough. */
+  if (length)
+    db= new_db;
+  else
+    db.clear();
 
   return false;
 }
 
-void Session::clear_db()
-{
-  db.clear();
-}
+
 
 
 /**

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2009-12-27 00:49:03 +0000
+++ drizzled/session.h	2010-01-22 00:53:13 +0000
@@ -1123,9 +1123,7 @@
       @retval false Success
       @retval true  Out-of-memory error
   */
-  bool set_db(const NormalisedDatabaseName &new_db);
-
-  void clear_db();
+  bool set_db(const char *new_db, size_t new_db_len);
 
   /*
     Copy the current database to the argument. Use the current arena to

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2010-01-09 00:51:36 +0000
+++ drizzled/show.cc	2010-01-22 00:53:13 +0000
@@ -303,18 +303,18 @@
   return false;
 }
 
-bool mysqld_show_create_db(Session *session, const NormalisedDatabaseName &database_name, bool if_not_exists)
+bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists)
 {
   char buff[2048];
   String buffer(buff, sizeof(buff), system_charset_info);
 
-  if (store_db_create_info(database_name.to_string().c_str(), &buffer, if_not_exists))
+  if (store_db_create_info(dbname, &buffer, if_not_exists))
   {
     /*
       This assumes that the only reason for which store_db_create_info()
       can fail is incorrect database name (which is the case now).
     */
-    my_error(ER_BAD_DB_ERROR, MYF(0), database_name.to_string().c_str());
+    my_error(ER_BAD_DB_ERROR, MYF(0), dbname);
     return true;
   }
 
@@ -325,7 +325,7 @@
   if (session->client->sendFields(&field_list))
     return true;
 
-  session->client->store(database_name.to_string().c_str(), database_name.to_string().length());
+  session->client->store(dbname, strlen(dbname));
   session->client->store(buffer.ptr(), buffer.length());
 
   if (session->client->flush())

=== modified file 'drizzled/show.h'
--- drizzled/show.h	2009-12-21 08:20:42 +0000
+++ drizzled/show.h	2010-01-22 00:53:13 +0000
@@ -89,7 +89,7 @@
 void mysqld_list_fields(Session *session,TableList *table, const char *wild);
 int mysqld_dump_create_info(Session *session, TableList *table_list, int fd);
 bool drizzled_show_create(Session *session, TableList *table_list, bool is_if_not_exists);
-bool mysqld_show_create_db(Session *session, const NormalisedDatabaseName &database_name, bool if_not_exists);
+bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists);
 
 int mysqld_show_status(Session *session);
 int mysqld_show_variables(Session *session,const char *wild);

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2009-12-25 05:07:36 +0000
+++ drizzled/sql_parse.cc	2010-01-22 00:53:13 +0000
@@ -202,13 +202,11 @@
   switch (command) {
   case COM_INIT_DB:
   {
-    string database_name(packet);
-    NonNormalisedDatabaseName non_normalised_database_name(database_name);
-    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
+    LEX_STRING tmp;
     status_var_increment(session->status_var.com_stat[SQLCOM_CHANGE_DB]);
-
-    if (! mysql_change_db(session, normalised_database_name, false))
+    tmp.str= packet;
+    tmp.length= packet_length;
+    if (!mysql_change_db(session, &tmp, false))
     {
       session->my_ok();
     }
@@ -367,6 +365,7 @@
   if (schema_table_name.compare("TABLES") == 0 ||
       schema_table_name.compare("TABLE_NAMES") == 0)
   {
+    LEX_STRING db;
     size_t dummy;
     if (lex->select_lex.db == NULL &&
         lex->copy_db_to(&lex->select_lex.db, &dummy))
@@ -374,18 +373,13 @@
       return (1);
     }
     schema_select_lex= new Select_Lex();
-    schema_select_lex->db= lex->select_lex.db;
-
-    string database_name(schema_select_lex->db);
-    NonNormalisedDatabaseName non_normalised_database_name(database_name);
-    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
+    db.str= schema_select_lex->db= lex->select_lex.db;
     schema_select_lex->table_list.first= NULL;
+    db.length= strlen(db.str);
 
-    if (! normalised_database_name.isValid())
+    if (check_db_name(&db))
     {
-      my_error(ER_WRONG_DB_NAME, MYF(0),
-               normalised_database_name.to_string().c_str());
+      my_error(ER_WRONG_DB_NAME, MYF(0), db.str);
       return (1);
     }
   }
@@ -958,20 +952,11 @@
     return NULL;
   }
 
-  if (table->is_derived_table() == false && table->db.str)
+  if (table->is_derived_table() == false && table->db.str &&
+      check_db_name(&table->db))
   {
-    string database_name(table->db.str);
-    NonNormalisedDatabaseName non_normalised_database_name(database_name);
-    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
-    if (! normalised_database_name.isValid())
-    {
-      my_error(ER_WRONG_DB_NAME, MYF(0), normalised_database_name.to_string().c_str());
-      return NULL;
-    }
-
-    strncpy(table->db.str, normalised_database_name.to_string().c_str(),
-            table->db.length);
+    my_error(ER_WRONG_DB_NAME, MYF(0), table->db.str);
+    return NULL;
   }
 
   if (!alias)					/* Alias is case sensitive */

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2009-12-27 00:49:03 +0000
+++ drizzled/sql_yacc.yy	2010-01-22 00:53:13 +0000
@@ -2439,17 +2439,6 @@
               DRIZZLE_YYABORT;
             }
 
-            if ($3->db.str)
-            {
-              std::string database_name($3->db.str);
-              NonNormalisedDatabaseName non_normalised_database_name(database_name);
-              NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-              if (! normalised_database_name.isValid())
-              {
-                my_error(ER_WRONG_TABLE_NAME, MYF(0), $3->table.str);
-                DRIZZLE_YYABORT;
-              }
-            }
             lex->name= $3->table;
             statement->alter_info.flags.set(ALTER_RENAME);
           }

=== modified file 'drizzled/statement/alter_schema.cc'
--- drizzled/statement/alter_schema.cc	2009-12-21 09:40:31 +0000
+++ drizzled/statement/alter_schema.cc	2010-01-22 00:53:13 +0000
@@ -33,15 +33,14 @@
 
 bool statement::AlterSchema::execute()
 {
-  string database_name(session->lex->name.str);
-  NonNormalisedDatabaseName non_normalised_database_name(database_name);
-  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+  LEX_STRING *db= &session->lex->name;
 
-  if (! normalised_database_name.isValid())
+  if (check_db_name(db))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), normalised_database_name.to_string().c_str());
+    my_error(ER_WRONG_DB_NAME, MYF(0), db->str);
     return false;
   }
+
   if (session->inTransaction())
   {
     my_message(ER_LOCK_OR_ACTIVE_TRANSACTION, 
@@ -49,7 +48,9 @@
                MYF(0));
     return true;
   }
-  bool res= mysql_alter_db(session, normalised_database_name, &schema_message);
+
+  bool res= mysql_alter_db(session, db->str, &schema_message);
+
   return res;
 }
 

=== modified file 'drizzled/statement/change_schema.cc'
--- drizzled/statement/change_schema.cc	2009-12-21 09:40:31 +0000
+++ drizzled/statement/change_schema.cc	2010-01-22 00:53:13 +0000
@@ -34,11 +34,9 @@
 bool statement::ChangeSchema::execute()
 {
   Select_Lex *select_lex= &session->lex->select_lex;
-  string database_name(select_lex->db);
-  NonNormalisedDatabaseName non_normalised_database_name(database_name);
-  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+  LEX_STRING db_str= { (char *) select_lex->db, strlen(select_lex->db) };
 
-  if (! mysql_change_db(session, normalised_database_name, false))
+  if (! mysql_change_db(session, &db_str, false))
   {
     session->my_ok();
   }

=== modified file 'drizzled/statement/create_schema.cc'
--- drizzled/statement/create_schema.cc	2009-12-21 09:40:31 +0000
+++ drizzled/statement/create_schema.cc	2010-01-22 00:53:13 +0000
@@ -33,22 +33,20 @@
 
 bool statement::CreateSchema::execute()
 {
-  string database_name(session->lex->name.str);
-  NonNormalisedDatabaseName non_normalised_database_name(database_name);
-  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
-
   if (! session->endActiveTransaction())
   {
     return true;
   }
-  if (! session->lex->name.str ||
-      ! normalised_database_name.isValid())
+  char *alias= session->strmake(session->lex->name.str,
+                                session->lex->name.length);
+  if (! alias ||
+      check_db_name(&session->lex->name))
   {
     my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
     return false;
   }
-  bool res= mysql_create_db(session, normalised_database_name, &schema_message, is_if_not_exists);
+
+  bool res= mysql_create_db(session, session->lex->name.str, &schema_message, is_if_not_exists);
   return res;
 }
 

=== modified file 'drizzled/statement/drop_schema.cc'
--- drizzled/statement/drop_schema.cc	2009-12-21 09:40:31 +0000
+++ drizzled/statement/drop_schema.cc	2010-01-22 00:53:13 +0000
@@ -33,18 +33,13 @@
 
 bool statement::DropSchema::execute()
 {
-  string database_name(session->lex->name.str);
-  NonNormalisedDatabaseName non_normalised_database_name(database_name);
-  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
   if (! session->endActiveTransaction())
   {
     return true;
   }
-  if (! normalised_database_name.isValid())
+  if (check_db_name(&session->lex->name))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0),
-             normalised_database_name.to_string().c_str());
+    my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
     return false;
   }
   if (session->inTransaction())
@@ -54,7 +49,7 @@
         MYF(0));
     return true;
   }
-  bool res= mysql_rm_db(session, normalised_database_name, drop_if_exists);
+  bool res= mysql_rm_db(session, session->lex->name.str, drop_if_exists);
   return res;
 }
 

=== modified file 'drizzled/statement/show_create_schema.cc'
--- drizzled/statement/show_create_schema.cc	2009-12-21 09:40:31 +0000
+++ drizzled/statement/show_create_schema.cc	2010-01-22 00:53:13 +0000
@@ -33,17 +33,12 @@
 
 bool statement::ShowCreateSchema::execute()
 {
-  string database_name(session->lex->name.str);
-  NonNormalisedDatabaseName non_normalised_database_name(database_name);
-  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
-
-  if (! normalised_database_name.isValid())
+  if (check_db_name(&session->lex->name))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0),
-             normalised_database_name.to_string().c_str());
+    my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
     return false;
   }
-  bool res= mysqld_show_create_db(session, normalised_database_name,
+  bool res= mysqld_show_create_db(session, session->lex->name.str,
                                   is_if_not_exists);
   return res;
 }

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2010-01-09 01:51:11 +0000
+++ drizzled/table.cc	2010-01-22 00:53:13 +0000
@@ -1927,6 +1927,31 @@
 }
 
 /*
+  Check if database name is valid
+
+  SYNPOSIS
+    check_db_name()
+    org_name		Name of database and length
+
+  RETURN
+    0	ok
+    1   error
+*/
+
+bool check_db_name(LEX_STRING *org_name)
+{
+  char *name= org_name->str;
+  uint32_t name_length= org_name->length;
+
+  if (!name_length || name_length > NAME_LEN || name[name_length - 1] == ' ')
+    return 1;
+
+  my_casedn_str(files_charset_info, name);
+
+  return check_identifier_name(org_name);
+}
+
+/*
   Allow anything as a table name, as long as it doesn't contain an
   ' ' at the end
   returns 1 on error

=== modified file 'drizzled/table.h'
--- drizzled/table.h	2010-01-09 01:51:11 +0000
+++ drizzled/table.h	2010-01-22 00:53:13 +0000
@@ -552,6 +552,7 @@
 
 int rename_file_ext(const char * from,const char * to,const char * ext);
 bool check_column_name(const char *name);
+bool check_db_name(LEX_STRING *org_name);
 bool check_table_name(const char *name, uint32_t length);
 
 #endif /* DRIZZLED_TABLE_H */

=== modified file 'tests/r/bigint.result'
--- tests/r/bigint.result	2009-10-31 18:12:20 +0000
+++ tests/r/bigint.result	2010-01-22 00:53:13 +0000
@@ -183,7 +183,7 @@
 drop table t1;
 DROP DATABASE IF EXISTS `scott`;
 Warnings:
-Note	1008	Can't drop database 'scott'; database doesn't exist
+Note	1008	Can't drop database './scott/'; database doesn't exist
 create table t1 (a char(100), b varchar(100), c text, d blob);
 insert into t1 values(
 18446744073709551615,18446744073709551615,

=== modified file 'tests/r/create.result'
--- tests/r/create.result	2009-12-01 18:40:22 +0000
+++ tests/r/create.result	2010-01-22 00:53:13 +0000
@@ -76,7 +76,7 @@
 drop table t1;
 drop database if exists mysqltest;
 Warnings:
-Note	1008	Can't drop database 'mysqltest'; database doesn't exist
+Note	1008	Can't drop database './mysqltest/'; database doesn't exist
 create database mysqltest;
 create table mysqltest.$test1 (a$1 int, $b int, c$ int);
 insert into mysqltest.$test1 values (1,2,3);
@@ -539,7 +539,7 @@
 create database mysqltest;
 create database if not exists mysqltest;
 Warnings:
-Note	1007	Can't create database 'mysqltest'; database exists
+Note	1007	Can't create database './mysqltest'; database exists
 show create database mysqltest;
 Database	Create Database
 mysqltest	CREATE DATABASE `mysqltest`

=== modified file 'tests/r/drop.result'
--- tests/r/drop.result	2009-06-05 23:10:06 +0000
+++ tests/r/drop.result	2010-01-22 00:53:13 +0000
@@ -61,7 +61,7 @@
 mysql
 test
 drop database mysqltest;
-ERROR HY000: Can't drop database 'mysqltest'; database doesn't exist
+ERROR HY000: Can't drop database './mysqltest/'; database doesn't exist
 drop table t1;
 flush tables with read lock;
 create table t1(n int);

=== modified file 'tests/r/lock_multi.result'
--- tests/r/lock_multi.result	2009-06-05 23:10:06 +0000
+++ tests/r/lock_multi.result	2010-01-22 00:53:13 +0000
@@ -6,5 +6,5 @@
 ERROR HY000: Can't execute the query because you have a conflicting read lock
 UNLOCK TABLES;
 DROP DATABASE mysqltest_1;
-ERROR HY000: Can't drop database 'mysqltest_1'; database doesn't exist
+ERROR HY000: Can't drop database './mysqltest_1/'; database doesn't exist
 End of 5.1 tests

=== modified file 'tests/r/lowercase_fs_off.result'
--- tests/r/lowercase_fs_off.result	2008-12-16 17:34:59 +0000
+++ tests/r/lowercase_fs_off.result	2010-01-22 00:53:13 +0000
@@ -4,6 +4,6 @@
 d1
 create database d2;
 create database D1;
-ERROR HY000: Can't create database 'd1'; database exists
+ERROR HY000: Can't create database './d1'; database exists
 drop database d2;
 drop database d1;

=== modified file 'tests/r/warnings.result'
--- tests/r/warnings.result	2008-12-19 21:59:48 +0000
+++ tests/r/warnings.result	2010-01-22 00:53:13 +0000
@@ -43,7 +43,7 @@
 Note	1051	Unknown table 'not_exists_table'
 drop database if exists not_exists_db;
 Warnings:
-Note	1008	Can't drop database 'not_exists_db'; database doesn't exist
+Note	1008	Can't drop database './not_exists_db/'; database doesn't exist
 show count(*) warnings;
 @@session.warning_count
 1

